( cd usr/lib ; rm -rf libxerces-c.so.28 )
( cd usr/lib ; ln -sf libxerces-c.so.28.0 libxerces-c.so.28 )
( cd usr/lib ; rm -rf libxerces-c.so )
( cd usr/lib ; ln -sf libxerces-c.so.28.0 libxerces-c.so )
( cd usr/lib ; rm -rf libxerces-depdom.so )
( cd usr/lib ; ln -sf libxerces-depdom.so.28.0 libxerces-depdom.so )
( cd usr/lib ; rm -rf libxerces-depdom.so.28 )
( cd usr/lib ; ln -sf libxerces-depdom.so.28.0 libxerces-depdom.so.28 )
