#ifndef DOM_DEPRECATED_HEADER_GUARD_
#define DOM_DEPRECATED_HEADER_GUARD_


/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Id: DOM.hpp 568078 2007-08-21 11:43:25Z amassari $
 */

//
//  This is the primary header file for inclusion in application
//  programs using the C++ XML Document Object Model API.
//

#include "DOM_Attr.hpp"
#include "DOM_CDATASection.hpp"
#include "DOM_CharacterData.hpp"
#include "DOM_Comment.hpp"
#include "DOM_Document.hpp"
#include "DOM_DocumentFragment.hpp"
#include "DOM_DocumentType.hpp"
#include "DOM_DOMException.hpp"
#include "DOM_DOMImplementation.hpp"
#include "DOM_Element.hpp"
#include "DOM_Entity.hpp"
#include "DOM_EntityReference.hpp"
#include "DOM_NamedNodeMap.hpp"
#include "DOM_Node.hpp"
#include "DOM_NodeList.hpp"
#include "DOM_Notation.hpp"
#include "DOM_ProcessingInstruction.hpp"
#include "DOM_Text.hpp"
#include "DOMString.hpp"
#include "DOM_XMLDecl.hpp"


#endif
